@@ -24,19 +24,17 @@
  * ```
  */
 #![allow(clippy::field_reassign_with_default)]
-use std::borrow::Cow;
-use std::collections::HashMap;
-use std::env;
-use std::error;
-use std::fmt;
-use std::fmt::Debug;
-use std::sync::Arc;
+use std::{
+    borrow::Cow, collections::HashMap, env, error, fmt, fmt::Debug, str::FromStr, sync::Arc,
+};
 
-use chrono::offset::Utc;
-use chrono::DateTime;
+use chrono::{offset::Utc, DateTime};
 use reqwest::{header, Client, Method, Request, StatusCode, Url};
 use schemars::JsonSchema;
-use serde::{Deserialize, Serialize};
+use serde::{
+    de::{self, Visitor},
+    Deserialize, Serialize,
+};
 
 /// Endpoint for the Shippo API.
 const ENDPOINT: &str = "https://api.goshippo.com/";
@@ -77,7 +75,13 @@ impl Shippo {
         Shippo::new(token)
     }
 
-    fn request<B>(&self, method: Method, path: &str, body: B, query: Option<Vec<(String, String)>>) -> Request
+    fn request<B>(
+        &self,
+        method: Method,
+        path: &str,
+        body: B,
+        query: Option<Vec<(String, String)>>,
+    ) -> Request
     where
         B: Serialize,
     {
@@ -90,7 +94,10 @@ impl Shippo {
         // Set the default headers.
         let mut headers = header::HeaderMap::new();
         headers.append(header::AUTHORIZATION, bearer);
-        headers.append(header::CONTENT_TYPE, header::HeaderValue::from_static("application/json"));
+        headers.append(
+            header::CONTENT_TYPE,
+            header::HeaderValue::from_static("application/json"),
+        );
 
         let mut rb = self.client.request(method.clone(), url).headers(headers);
 
@@ -154,6 +161,67 @@ impl Shippo {
         Ok(resp.json().await.unwrap())
     }
 
+    /// List the orders.
+    /// FROM: https://goshippo.com/docs/reference#orders-list
+    pub async fn list_orders(&self) -> Result<Vec<Order>, APIError> {
+        // Build the request.
+        let mut request = self.request(Method::GET, "orders", (), None);
+
+        let mut resp = self.client.execute(request).await.unwrap();
+        match resp.status() {
+            StatusCode::OK => (),
+            s => {
+                return Err(APIError {
+                    status_code: s,
+                    body: resp.text().await.unwrap(),
+                })
+            }
+        };
+
+        let mut r: OrdersAPIResponse = resp.json().await.unwrap();
+        let mut orders = r.orders;
+        let mut page = r.next;
+
+        // Paginate if we should.
+        // TODO: make this more DRY
+        while !page.is_empty() {
+            let url = Url::parse(&page).unwrap();
+            let pairs: Vec<(Cow<'_, str>, Cow<'_, str>)> = url.query_pairs().collect();
+            let mut new_pairs: Vec<(String, String)> = Vec::new();
+            for (a, b) in pairs {
+                let sa = a.into_owned();
+                let sb = b.into_owned();
+                new_pairs.push((sa, sb));
+            }
+
+            request = self.request(Method::GET, "orders", (), Some(new_pairs));
+
+            resp = self.client.execute(request).await.unwrap();
+            match resp.status() {
+                StatusCode::OK => (),
+                s => {
+                    return Err(APIError {
+                        status_code: s,
+                        body: resp.text().await.unwrap(),
+                    })
+                }
+            };
+
+            // Try to deserialize the response.
+            r = resp.json().await.unwrap();
+
+            orders.append(&mut r.orders);
+
+            if !r.next.is_empty() && r.next != page {
+                page = r.next;
+            } else {
+                page = "".to_string();
+            }
+        }
+
+        Ok(orders)
+    }
+
     /// List the carrier accounts.
     /// FROM: https://goshippo.com/docs/reference#carrier-accounts
     pub async fn list_carrier_accounts(&self) -> Result<Vec<CarrierAccount>, APIError> {
@@ -277,7 +345,10 @@ impl Shippo {
 
     /// Create a shipping label based on a rate.
     /// FROM: https://goshippo.com/docs/reference#transactions-create
-    pub async fn create_shipping_label_from_rate(&self, nt: NewTransaction) -> Result<Transaction, APIError> {
+    pub async fn create_shipping_label_from_rate(
+        &self,
+        nt: NewTransaction,
+    ) -> Result<Transaction, APIError> {
         // Build the request.
         let request = self.request(Method::POST, "transactions", nt, None);
 
@@ -354,7 +425,11 @@ impl Shippo {
     /// by POSTing to the tracking endpoint. This way Shippo will send HTTP notifications to your
     /// track_updated webhook(s) whenever the status changes.
     /// FROM: https://goshippo.com/docs/reference#tracks-create
-    pub async fn register_tracking_webhook(&self, carrier: &str, tracking_number: &str) -> Result<TrackingStatus, APIError> {
+    pub async fn register_tracking_webhook(
+        &self,
+        carrier: &str,
+        tracking_number: &str,
+    ) -> Result<TrackingStatus, APIError> {
         let mut body: HashMap<&str, &str> = HashMap::new();
         body.insert("tracking_number", tracking_number);
         body.insert("carrier", carrier);
@@ -379,9 +454,18 @@ impl Shippo {
 
     /// Request the tracking status of a shipment by sending a GET request.
     /// FROM: https://goshippo.com/docs/reference#tracks-retrieve
-    pub async fn get_tracking_status(&self, carrier: &str, tracking_number: &str) -> Result<TrackingStatus, APIError> {
+    pub async fn get_tracking_status(
+        &self,
+        carrier: &str,
+        tracking_number: &str,
+    ) -> Result<TrackingStatus, APIError> {
         // Build the request
-        let request = self.request(Method::GET, &format!("tracks/{}/{}", carrier, tracking_number), (), None);
+        let request = self.request(
+            Method::GET,
+            &format!("tracks/{}/{}", carrier, tracking_number),
+            (),
+            None,
+        );
 
         let resp = self.client.execute(request).await.unwrap();
         match resp.status() {
@@ -406,13 +490,23 @@ pub struct APIError {
 
 impl fmt::Display for APIError {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "APIError: status code -> {}, body -> {}", self.status_code.to_string(), self.body)
+        write!(
+            f,
+            "APIError: status code -> {}, body -> {}",
+            self.status_code.to_string(),
+            self.body
+        )
     }
 }
 
 impl fmt::Debug for APIError {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "APIError: status code -> {}, body -> {}", self.status_code.to_string(), self.body)
+        write!(
+            f,
+            "APIError: status code -> {}, body -> {}",
+            self.status_code.to_string(),
+            self.body
+        )
     }
 }
 
@@ -427,21 +521,57 @@ impl error::Error for APIError {
 /// The data type for an API response.
 #[derive(Clone, Debug, Default, Serialize, Deserialize)]
 pub struct APIResponse {
-    #[serde(default, deserialize_with = "deserialize_null_string::deserialize", skip_serializing_if = "String::is_empty")]
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
     pub next: String,
-    #[serde(default, deserialize_with = "deserialize_null_string::deserialize", skip_serializing_if = "String::is_empty")]
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
     pub previous: String,
 
     #[serde(default, skip_serializing_if = "Vec::is_empty", alias = "results")]
     pub shipments: Vec<Shipment>,
 }
 
+/// The data type for an API response.
+#[derive(Clone, Debug, Default, Serialize, Deserialize)]
+pub struct OrdersAPIResponse {
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
+    pub next: String,
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
+    pub previous: String,
+
+    #[serde(default, skip_serializing_if = "Vec::is_empty", alias = "results")]
+    pub orders: Vec<Order>,
+}
+
 /// The data type for an API response for carrier accounts.
 #[derive(Clone, Debug, Default, Serialize, Deserialize)]
 pub struct CarrierAccountsAPIResponse {
-    #[serde(default, deserialize_with = "deserialize_null_string::deserialize", skip_serializing_if = "String::is_empty")]
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
     pub next: String,
-    #[serde(default, deserialize_with = "deserialize_null_string::deserialize", skip_serializing_if = "String::is_empty")]
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
     pub previous: String,
 
     #[serde(default, skip_serializing_if = "Vec::is_empty", alias = "results")]
@@ -451,9 +581,17 @@ pub struct CarrierAccountsAPIResponse {
 /// The data type for a transactions API response.
 #[derive(Clone, Debug, Default, Serialize, Deserialize)]
 pub struct TransactionsAPIResponse {
-    #[serde(default, deserialize_with = "deserialize_null_string::deserialize", skip_serializing_if = "String::is_empty")]
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
     pub next: String,
-    #[serde(default, deserialize_with = "deserialize_null_string::deserialize", skip_serializing_if = "String::is_empty")]
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
     pub previous: String,
 
     #[serde(default, skip_serializing_if = "Vec::is_empty", alias = "results")]
@@ -604,7 +742,11 @@ pub struct Address {
     #[serde(default, skip_serializing_if = "String::is_empty")]
     pub email: String,
     /// Indicates whether the object has been created in test mode.
-    #[serde(default, skip_serializing_if = "is_false")]
+    #[serde(
+        default,
+        skip_serializing_if = "is_false",
+        deserialize_with = "deserialize_null_boolean::deserialize"
+    )]
     pub test: bool,
     /// object that contains information regarding if an address had been validated or not. Also
     /// contains any messages generated during validation. Children keys are is_valid(boolean) and
@@ -624,11 +766,18 @@ impl Address {
         if self.country == "US" && zip.len() > 5 {
             zip.insert(5, '-');
         }
-        format!("{}\n{}, {} {} {}", street.trim(), self.city, self.state, zip, self.country)
-            .trim()
-            .trim_matches(',')
-            .trim()
-            .to_string()
+        format!(
+            "{}\n{}, {} {} {}",
+            street.trim(),
+            self.city,
+            self.state,
+            zip,
+            self.country
+        )
+        .trim()
+        .trim_matches(',')
+        .trim()
+        .to_string()
     }
 }
 
@@ -766,13 +915,25 @@ pub struct ServiceLevel {
     /// A servicelevel commonly defines the transit time of a Shipment
     /// (e.g., Express vs. Standard), along with other properties.
     /// These names vary depending on the provider.
-    #[serde(default, skip_serializing_if = "String::is_empty", deserialize_with = "deserialize_null_string::deserialize")]
+    #[serde(
+        default,
+        skip_serializing_if = "String::is_empty",
+        deserialize_with = "deserialize_null_string::deserialize"
+    )]
     pub name: String,
     /// Token of the Rate's servicelevel, e.g. "usps_priority" or "fedex_ground".
-    #[serde(default, skip_serializing_if = "String::is_empty", deserialize_with = "deserialize_null_string::deserialize")]
+    #[serde(
+        default,
+        skip_serializing_if = "String::is_empty",
+        deserialize_with = "deserialize_null_string::deserialize"
+    )]
     pub token: String,
     /// Further clarification of the service.
-    #[serde(default, skip_serializing_if = "String::is_empty", deserialize_with = "deserialize_null_string::deserialize")]
+    #[serde(
+        default,
+        skip_serializing_if = "String::is_empty",
+        deserialize_with = "deserialize_null_string::deserialize"
+    )]
     pub terms: String,
 }
 
@@ -847,8 +1008,8 @@ pub struct Pickup {
     #[serde(default, skip_serializing_if = "String::is_empty")]
     pub timezone: String,
     /// An array containing strings of any messages generated during validation.
-    #[serde(default, skip_serializing_if = "Vec::is_empty")]
-    pub messages: Vec<Message>,
+    #[serde(default, skip_serializing_if = "Option::is_none")]
+    pub messages: Option<Vec<Message>>,
     /// A string of up to 100 characters that can be filled with any additional
     /// information you want to attach to the object.
     #[serde(default, skip_serializing_if = "String::is_empty")]
@@ -907,7 +1068,8 @@ pub struct Transaction {
     #[serde(default, skip_serializing_if = "String::is_empty")]
     pub object_id: String,
     /// Date and time of Transaction creation.
-    pub object_created: DateTime<Utc>,
+    #[serde(default, skip_serializing_if = "Option::is_none")]
+    pub object_created: Option<DateTime<Utc>>,
     /// Date and time of last Transaction update.
     #[serde(default, skip_serializing_if = "Option::is_none")]
     pub object_updated: Option<DateTime<Utc>>,
@@ -949,7 +1111,11 @@ pub struct Transaction {
     #[serde(default, skip_serializing_if = "String::is_empty")]
     pub tracking_url_provider: String,
     /// The estimated time of arrival according to the carrier.
-    #[serde(deserialize_with = "null_date_format::deserialize", skip_serializing_if = "Option::is_none")]
+    #[serde(
+        default,
+        deserialize_with = "null_date_format::deserialize",
+        skip_serializing_if = "Option::is_none"
+    )]
     pub eta: Option<DateTime<Utc>>,
     /// A URL pointing directly to the label in the format you've set in your settings.
     /// A value will only be returned if the Transactions has been processed successfully.
@@ -957,7 +1123,11 @@ pub struct Transaction {
     pub label_url: String,
     /// A URL pointing to the commercial invoice as a 8.5x11 inch PDF file.
     /// A value will only be returned if the Transactions has been processed successfully and if the shipment is international.
-    #[serde(default, skip_serializing_if = "String::is_empty", deserialize_with = "deserialize_null_string::deserialize")]
+    #[serde(
+        default,
+        skip_serializing_if = "String::is_empty",
+        deserialize_with = "deserialize_null_string::deserialize"
+    )]
     pub commercial_invoice_url: String,
     /// An array containing elements of the following schema:
     /// - "code" (string): an identifier for the corresponding message (not always available")
@@ -966,7 +1136,11 @@ pub struct Transaction {
     pub messages: Vec<Message>,
     /// A URL pointing directly to the QR code in PNG format.
     /// A value will only be returned if requested using qr_code_requested flag and the carrier provides such an option.
-    #[serde(default, skip_serializing_if = "String::is_empty", deserialize_with = "deserialize_null_string::deserialize")]
+    #[serde(
+        default,
+        skip_serializing_if = "String::is_empty",
+        deserialize_with = "deserialize_null_string::deserialize"
+    )]
     pub qr_code_url: String,
     /// Indicates whether the object has been created in test mode.
     #[serde(default)]
@@ -1011,10 +1185,18 @@ pub struct ValidationResults {
 #[derive(Clone, Debug, Default, JsonSchema, Serialize, Deserialize)]
 pub struct TrackingStatus {
     /// Name of the carrier of the shipment to track.
-    #[serde(default, deserialize_with = "deserialize_null_string::deserialize", skip_serializing_if = "String::is_empty")]
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
     pub carrier: String,
     /// Tracking number to track.
-    #[serde(default, deserialize_with = "deserialize_null_string::deserialize", skip_serializing_if = "String::is_empty")]
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
     pub tracking_number: String,
     /// The sender address with city, state, zip and country information.
     #[serde(default)]
@@ -1024,15 +1206,25 @@ pub struct TrackingStatus {
     pub address_to: Option<Address>,
     /// The object_id of the transaction associated with this tracking object.
     /// This field is visible only to the object owner of the transaction.
-    #[serde(default, deserialize_with = "deserialize_null_string::deserialize", skip_serializing_if = "String::is_empty")]
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
     pub transaction: String,
     /// The estimated time of arrival according to the carrier, this might be
     /// updated by carriers during the life of the shipment.
-    #[serde(deserialize_with = "null_date_format::deserialize", skip_serializing_if = "Option::is_none")]
+    #[serde(
+        deserialize_with = "null_date_format::deserialize",
+        skip_serializing_if = "Option::is_none"
+    )]
     pub eta: Option<DateTime<Utc>>,
     /// The estimated time of arrival according to the carrier at the time the
     /// shipment first entered the system.
-    #[serde(deserialize_with = "null_date_format::deserialize", skip_serializing_if = "Option::is_none")]
+    #[serde(
+        deserialize_with = "null_date_format::deserialize",
+        skip_serializing_if = "Option::is_none"
+    )]
     pub original_eta: Option<DateTime<Utc>>,
     /// The service level of the shipment as token and full name.
     #[serde(default)]
@@ -1045,7 +1237,11 @@ pub struct TrackingStatus {
     pub tracking_history: Vec<Status>,
     /// A string of up to 100 characters that can be filled with any additional information you
     /// want to attach to the object.
-    #[serde(default, deserialize_with = "deserialize_null_string::deserialize", skip_serializing_if = "String::is_empty")]
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
     pub metadata: String,
 }
 
@@ -1053,14 +1249,25 @@ pub struct TrackingStatus {
 pub struct Status {
     /// Indicates the high level status of the shipment.
     /// 'UNKNOWN' | 'PRE_TRANSIT' | 'TRANSIT' | 'DELIVERED' | 'RETURNED' | 'FAILURE'
-    #[serde(default, deserialize_with = "deserialize_null_string::deserialize", skip_serializing_if = "String::is_empty")]
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
     pub status: String,
     /// The human-readable description of the status.
-    #[serde(default, deserialize_with = "deserialize_null_string::deserialize", skip_serializing_if = "String::is_empty")]
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
     pub status_details: String,
     /// Date and time when the carrier scanned this tracking event.
     /// This is displayed in UTC.
-    #[serde(deserialize_with = "null_date_format::deserialize", skip_serializing_if = "Option::is_none")]
+    #[serde(
+        deserialize_with = "null_date_format::deserialize",
+        skip_serializing_if = "Option::is_none"
+    )]
     pub status_date: Option<DateTime<Utc>>,
     /// An object containing zip, city, state and country information of the tracking event.
     #[serde(default, skip_serializing_if = "Option::is_none")]
@@ -1069,13 +1276,29 @@ pub struct Status {
 
 #[derive(Clone, Debug, Default, JsonSchema, Serialize, Deserialize)]
 pub struct TrackingLocation {
-    #[serde(default, deserialize_with = "deserialize_null_string::deserialize", skip_serializing_if = "String::is_empty")]
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
     pub city: String,
-    #[serde(default, deserialize_with = "deserialize_null_string::deserialize", skip_serializing_if = "String::is_empty")]
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
     pub state: String,
-    #[serde(default, deserialize_with = "deserialize_null_string::deserialize", skip_serializing_if = "String::is_empty")]
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
     pub zip: String,
-    #[serde(default, deserialize_with = "deserialize_null_string::deserialize", skip_serializing_if = "String::is_empty")]
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
     pub country: String,
 }
 
@@ -1085,7 +1308,11 @@ impl TrackingLocation {
         if self.country == "US" && zip.len() > 5 {
             zip.insert(5, '-');
         }
-        format!("{}, {} {} {}", self.city, self.state, zip, self.country).trim().trim_matches(',').trim().to_string()
+        format!("{}, {} {} {}", self.city, self.state, zip, self.country)
+            .trim()
+            .trim_matches(',')
+            .trim()
+            .to_string()
     }
 }
 
@@ -1174,6 +1401,105 @@ pub struct CustomsDeclaration {
     pub test: bool,
 }
 
+/// An order object.
+/// FROM: https://goshippo.com/docs/reference#orders
+#[derive(Clone, Debug, Serialize, Deserialize)]
+pub struct Order {
+    /// Unique identifier of the given object.
+    #[serde(default, skip_serializing_if = "String::is_empty")]
+    pub object_id: String,
+    /// Username of the user who created the object.
+    #[serde(default, skip_serializing_if = "String::is_empty")]
+    pub object_owner: String,
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
+    pub order_number: String,
+    pub placed_at: DateTime<Utc>,
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
+    pub order_status: String,
+    #[serde(default)]
+    pub to_address: Address,
+    #[serde(default)]
+    pub from_address: Address,
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
+    pub shop_app: String,
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
+    pub weight: String,
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
+    pub weight_unit: String,
+    #[serde(default, skip_serializing_if = "Vec::is_empty")]
+    pub transactions: Vec<Transaction>,
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
+    pub total_tax: String,
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
+    pub total_price: String,
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
+    pub subtotal_price: String,
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
+    pub currency: String,
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
+    pub shipping_method: String,
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
+    pub shipping_cost: String,
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
+    pub shipping_cost_currency: String,
+    #[serde(
+        default,
+        deserialize_with = "deserialize_null_string::deserialize",
+        skip_serializing_if = "String::is_empty"
+    )]
+    pub notes: String,
+    #[serde(default)]
+    pub test: bool,
+}
+
 /// A customs item object.
 /// Customs items are distinct items in your international shipment parcel.
 /// FROM: https://goshippo.com/docs/reference#customs-items
@@ -1214,7 +1540,11 @@ pub struct CustomsItem {
     #[serde(default, skip_serializing_if = "String::is_empty")]
     pub tariff_number: String,
     /// SKU code of the item, which is required by some carriers.
-    #[serde(default, skip_serializing_if = "String::is_empty", deserialize_with = "deserialize_null_string::deserialize")]
+    #[serde(
+        default,
+        skip_serializing_if = "String::is_empty",
+        deserialize_with = "deserialize_null_string::deserialize"
+    )]
     pub sku_code: String,
     /// Export Control Classification Number, required on some exports from the United States.
     #[serde(default, skip_serializing_if = "String::is_empty")]
@@ -1270,3 +1600,68 @@ pub mod null_date_format {
         Ok(Some(Utc.datetime_from_str(&s, "%+").unwrap()))
     }
 }
+
+pub mod deserialize_null_boolean {
+    use serde::{self, Deserializer};
+
+    // The signature of a deserialize_with function must follow the pattern:
+    //
+    //    fn deserialize<'de, D>(D) -> Result<T, D::Error>
+    //    where
+    //        D: Deserializer<'de>
+    //
+    // although it may also be generic over the output types T.
+    pub fn deserialize<'de, D>(deserializer: D) -> Result<bool, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        let s = deserializer
+            .deserialize_bool(crate::BoolVisitor)
+            .unwrap_or_default();
+
+        Ok(s)
+    }
+}
+
+struct BoolVisitor;
+
+impl<'de> Visitor<'de> for BoolVisitor {
+    type Value = bool;
+
+    fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+        formatter.write_str("a boolean")
+    }
+
+    fn visit_bool<E>(self, value: bool) -> Result<Self::Value, E>
+    where
+        E: de::Error,
+    {
+        Ok(value)
+    }
+
+    fn visit_str<E>(self, value: &str) -> Result<Self::Value, E>
+    where
+        E: de::Error,
+    {
+        match FromStr::from_str(value) {
+            Ok(s) => Ok(s),
+            Err(_) => Err(de::Error::invalid_value(
+                de::Unexpected::Str(value),
+                &"bool",
+            )),
+        }
+    }
+
+    fn visit_string<E>(self, value: String) -> Result<Self::Value, E>
+    where
+        E: de::Error,
+    {
+        match FromStr::from_str(&value) {
+            Ok(s) => Ok(s),
+            Err(_) => Err(de::Error::invalid_value(
+                de::Unexpected::Str(&value),
+                &"bool",
+            )),
+        }
+    }
+}
